#include "EGL/egl.h"
#include "EGL/eglext.h"
#include "GLES/gl.h"
#include <assert.h>
#include <iostream>

using namespace std;

//
//  Abfrage der Eigenschaften von EGL
//  Munshi, Aaftab; Ginsburg, Dan; Shreiner, Dave:
//  OpenGL ES 2.0 Programming Guide
//  Addison-Wesley, July 2008
//  ISBN-13:  978-0-321-50279-7
//  ISBN-10:      0-321-50279-5
//   Seite 38-39
//

void describeOpenGLES11();

int main(int  argc, char *argv[]) {
    EGLint majorVersion;
    EGLint minorVersion;
    EGLConfig config;
    EGLDisplay display;
    EGLSurface surface;
    EGLContext context;
    EGLBoolean result;
    EGLint numConfigs;
        static const EGLint attribute_list[] =
    {
      EGL_RED_SIZE, 8,
      EGL_GREEN_SIZE, 8,
      EGL_BLUE_SIZE, 8,
      EGL_ALPHA_SIZE, 8,
      EGL_DEPTH_SIZE, 24,
      EGL_RENDERABLE_TYPE, EGL_OPENGL_ES_BIT,
      EGL_SURFACE_TYPE, EGL_PIXMAP_BIT,
      EGL_NONE
    };
   EGLint context_attribs [] = { EGL_CONTEXT_CLIENT_VERSION, 1, EGL_NONE }; //  fuer OpenGL ES 1.1
   EGLint bufferAttribList [] = {
       EGL_WIDTH,  (EGLint)10,
       EGL_HEIGHT, (EGLint)10,
       EGL_LARGEST_PBUFFER, EGL_TRUE,
       EGL_NONE
    };
   
    
    cout << "Examination of EGL" << endl;
    display = eglGetDisplay(EGL_DEFAULT_DISPLAY);
    if (display == EGL_NO_DISPLAY) {
        cout << "unable to open connection to EGL display" << endl;
        exit (1);
    }
    if (!eglInitialize(display, &majorVersion, &minorVersion)) {
        cout << "Unable to initialize EGL" << endl;
        exit (1);
    } else {
        cout << "EGL version " << majorVersion << ":" << minorVersion << endl;
    }
    
    //  erste Anfrage: Anzahl der Konfigurationen erfragen
    result = eglGetConfigs (display, NULL, 20, &numConfigs);
    if (result != EGL_TRUE) {
        cout << "failure in eglGetConfigs" << endl;
        exit(1);
    }
    cout << "number of available configurations: " << numConfigs << endl;
    
    //  zweite Anfrage: alle Konfigurationen erfragen
    EGLConfig configs [numConfigs];
    EGLint nc;
    result = eglGetConfigs (display, &configs[0], numConfigs, &nc);
    
	result = eglChooseConfig(display,
	                         attribute_list,
	                         &config,
	                         1, 
	                         &numConfigs);
	assert(EGL_FALSE != result);
	eglBindAPI(EGL_OPENGL_ES_API);

	context = eglCreateContext(display,
	                           config,
	                           EGL_NO_CONTEXT,
	                           context_attribs);
	assert(EGL_NO_CONTEXT != context);
	
	
    surface = eglCreatePbufferSurface(display,
                                      config,
                                      bufferAttribList);
    assert(surface != EGL_NO_SURFACE);

    // connect the context to the surface
    result = eglMakeCurrent(display,
                            surface,
                            surface,
                            context);
    assert(EGL_FALSE != result);
    
	cout << endl << "Eigenschaften von OpenGL ES 1.1" << endl
	             << "-------------------------------" << endl;
    describeOpenGLES11();
    
	cout << "--------------------------------" << endl;
	
    eglMakeCurrent(display,
                   EGL_NO_SURFACE,
                   EGL_NO_SURFACE,
                   EGL_NO_CONTEXT);
    eglDestroySurface(display, surface);
    eglDestroyContext(display, context);
    eglTerminate(display);
	eglReleaseThread();
    return 0;
}

void describeOpenGLES11() {
	GLint intValues[8];

	cout << "GL_VENDOR      " << glGetString(GL_VENDOR) << endl;
    cout << "GL_RENDERER    " << glGetString(GL_RENDERER) << endl;
    cout << "GL_VERSION     " << glGetString(GL_VERSION) << endl;
    cout << "GL_EXTENSIONS  " << glGetString(GL_EXTENSIONS) << endl;

    glGetIntegerv(GL_MAX_CLIP_PLANES,  intValues);
    cout << "GL_MAX_CLIP_PLANES     " << intValues[0]  << endl;
    
    glGetIntegerv(GL_MAX_LIGHTS,  intValues);
    cout << "GL_MAX_LIGHTS     " << intValues[0]  << endl;
    
    glGetIntegerv(GL_MAX_MODELVIEW_STACK_DEPTH,  intValues);
    cout << "GL_MAX_MODELVIEW_STACK_DEPTH     " << intValues[0]  << endl;
    
    glGetIntegerv(GL_MAX_PROJECTION_STACK_DEPTH,  intValues);
    cout << "GL_MAX_PROJECTION_STACK_DEPTH     " << intValues[0]  << endl;
    
    glGetIntegerv(GL_MAX_TEXTURE_SIZE,  intValues);
    cout << "GL_MAX_TEXTURE_SIZE     " << intValues[0]  << endl;
    
    glGetIntegerv(GL_MAX_TEXTURE_STACK_DEPTH,  intValues);
    cout << "GL_MAX_TEXTURE_STACK_DEPTH     " << intValues[0]  << endl;
    
    glGetIntegerv(GL_MAX_TEXTURE_UNITS,  intValues);
    cout << "GL_MAX_TEXTURE_UNITS     " << intValues[0]  << endl;
    
    glGetIntegerv(GL_MAX_VIEWPORT_DIMS,  intValues);
    cout << "GL_MAX_VIEWPORT_DIMS     " << intValues[0] << ", " << intValues[1] << endl;
    
    glGetIntegerv(GL_NUM_COMPRESSED_TEXTURE_FORMATS,  intValues);
    cout << "GL_NUM_COMPRESSED_TEXTURE_FORMATS     " << intValues[0]  << endl;
}

